@@ -18,6 +18,9 @@ typedef struct dh_st {
     BIGNUM *priv_key;
     /* Public DH value g^x */
     BIGNUM *pub_key;
+    /* X9.42/RFC 2631 */
+    BIGNUM *q;
+    BIGNUM *j;
     ...;
 } DH;
 """
@@ -28,6 +31,7 @@ void DH_free(DH *);
 int DH_size(const DH *);
 DH *DH_generate_parameters(int, int, void (*)(int, int, void *), void *);
 int DH_check(const DH *, int *);
+int DH_check_pub_key(const DH *, const BIGNUM *, int *);
 int DH_generate_key(DH *);
 int DH_compute_key(unsigned char *, const BIGNUM *, DH *);
 int DH_set_ex_data(DH *, int, void *);
